classdef LeastSquares < falcon.parest.costfunction.impl.Base
    % least squares cost function for state and parameter estimation

    % -------------------------------------------------------------------------
    %                                FALCON.m
    % Copyright (c) 2014-2019 Institute of Flight System Dynamics, TUM, Munich
    % Matthias Bittner, Matthias Rieck, Maximilian Richter,
    % Benedikt Grueter, Johannes Diepolder, Florian Schwaiger,
    % Patrick Piprek, and Florian Holzapfel
    % Downloading, using, copying, or modifying FALCON.m code constitutes an
    % agreement to ALL of the terms of the FALCON.m EULA.
    % -------------------------------------------------------------------------

    properties
        % falcon.parest.costfunction.impl.LeastSquares/inv_Cov is a property.
        inv_Cov
        % falcon.parest.costfunction.impl.LeastSquares/Cov is a property.
        Cov
        % covariance matrix scaling
        CovarianceScalingMatrix
        % scalar scaling factor
        ScalarScaling
        % indicate if output scaling is to be used
        CovarianceScalingMode
    end

    methods
        function obj = LeastSquares(CostFunction)
        % least squares cost function for state and parameter estimation
        end

        function ddJ_ddp_val = ddJ_ddp(obj, meas, model, d_dp)
        % Hessian w.r.t. p
        %  
        % <Syntax>
        % [ ddJ_LS_ddp_val ] = ddJ_ddp( z, y, ScalingVector, dy_dp )
        %  
        % <Description> 
        % compute the matrix of second cost function derivatives with
        % respect to the parameters for the least squares cost function
        % <Inputs>
        % > z: cell array of measurements
        % > y: cell array of model outputs
        % > Scaling: cell containing scalar scaling value and
        % covariance scaling matrix
        % > dy_dp: output sensitivities w.r.t. the parameters
        %  
        % <Outputs> 
        % > ddJ_LS_ddp_val: value of the matrix of second cost
        % function derivative
        end

        function dJ_dp_val = dJ_dp(obj, meas, model, d_dp)
        % cost function derivative w.r.t. p
        %  
        % <Syntax>
        % [ dJ_LS_dp_val ] = dJ_dp( z, y, ScalingVector, dy_dp )
        %  
        % <Description>
        % compute the cost function derivatives with respect to the
        % parameters for the least squares cost function
        % <Inputs>
        % > z: cell array of measurements
        % > y: cell array of model outputs
        % > Scaling: cell containing scalar scaling value and
        % covariance scaling matrix
        % > dy_dp: output sensitivities w.r.t. the parameters
        %  
        % <Outputs>
        % > dJ_LS_dp_val: value of the cost function derivative
        end

        function ddJ_dd_val = ddJ_dd(obj, meas, model)
        % falcon.parest.costfunction.impl.LeastSquares/ddJ_dd is a function.
        %   [ddJ_dd_val] = ddJ_dd(obj, meas, model)
        end

        function dJ_d_val = dJ_d(obj, meas, model)
        % cost function derivative w.r.t. y
        %  
        % <Syntax>
        % [ dJ_LS_dy_val ] = dJ_dy( z, y, ScalingVector, SplineMatrix_fine, SplineMatrix_raw)
        %  
        % <Description>
        % compute the cost function derivatives with respect to the
        % model outputs for the least squares cost function
        % <Inputs>
        % > z: cell array of measurements
        % > y: cell array of model outputs
        % > Scaling: cell containing scalar scaling value and
        % covariance scaling matrix
        % > SplineMatrix_fine: matrix for spline interpolation 
        % > SplineMatrix_raw: matrix for spline interpolation 
        %  
        % <Outputs>
        % > dJ_LS_dy_val: value of cost function derivative
        end

        function [J_val, inv_Cov_loc] = J(obj, meas, model)
        % least squares cost function
        %  
        % <Syntax>
        % [ J_LS_val ] = obj.J( z, y )
        % [ J_LS_val ] = obj.J( z, y, u, w )
        %  
        % <Description>
        % compute the value of the least squares cost function
        % <Inputs>
        % > z: cell array of measurement values
        % > y: cell array of model output values
        % > Scaling: cell containing scalar scaling value and
        % covariance scaling matrix
        % <Outputs>
        % > J_LS_val: cost function value
        end

    end

end