classdef Experiment < falcon.core.Handle & falcon.core.HasToStruct & falcon.core.HasProblem & falcon.parest.plot.Experiment & falcon.parest.core.ProcNoiseExperiment & falcon.parest.core.hasSignalUncertainty
    % FALCON.PAREST.EXPERIMENT container for experiment data
    %  
    %  	<Description>
    %   in this class, the data from one experiment can be stored, i.e. the
    %   input and output data, together with the corresponding signal
    %   names.
    %  
    %   Furthermore, in order to characterize the experiments, the "Tags"
    %   property can be used with custom fields, where meta-data like
    %   maneuver name, flight date, aircraft configuration etc. can be
    %   stored. The falcon.parest.ExperimentFilter can then be used to
    %   select only certain experiments based on their "Tags".
    %  
    %   Extensive plot capabilities are implemented in
    %   falcon.parest.plot.Experiment.
    %  
    %  	<ChangeLog>
    %  	2017-12-19 - Ch. Goettlicher: commenting

    % -------------------------------------------------------------------------
    %                                FALCON.m
    % Copyright (c) 2014-2019 Institute of Flight System Dynamics, TUM, Munich
    % Matthias Bittner, Matthias Rieck, Maximilian Richter,
    % Benedikt Grueter, Johannes Diepolder, Florian Schwaiger,
    % Patrick Piprek, and Florian Holzapfel
    % Downloading, using, copying, or modifying FALCON.m code constitutes an
    % agreement to ALL of the terms of the FALCON.m EULA.
    % -------------------------------------------------------------------------

    properties
        % tags describing this experiment. can be used to filter for
        % certain experiments
        Tags
        % factor to decrease the integration sample time in order to reduce
        % computation time. Model outputs will be interpolated to fit the
        % measurement grid based on spline interpolation;
        % a factor < 1 is interpreted as percentage of original samples to
        % be kept. a factor > 1 is interpreted as explicit number of
        % samples to retain
        SampleTimeFactor
        % interpolation order for spline interpolation of model outputs to
        % measurement grid
        SplineInterpolationOrder
        % initial condition for this experiment
        x0
        % size of input vector
        nU
        % size of state vector
        nX
        % size of output vector
        nY
        % sample size of the states, sampled down by
        % falcon.parest.Experiment.SampleTimeFactor
        Nstate
        % sample size of the measurements
        Nmeasurement
        % numerical values of the states of the model
        StateValues
        % falcon.parest.Experiment/StateUpperBound is a property.
        StateUpperBound
        % falcon.parest.Experiment/StateLowerBound is a property.
        StateLowerBound
        % numericla values of the outputs of the model
        OutputValues
        % falcon.parest.Experiment/OutputUpperBound is a property.
        OutputUpperBound
        % falcon.parest.Experiment/OutputLowerBound is a property.
        OutputLowerBound
        % falcon.parest.Experiment/ControlUpperBound is a property.
        ControlUpperBound
        % falcon.parest.Experiment/ControlLowerBound is a property.
        ControlLowerBound
        % average sample time
        MeanSampleTime
        % numerical values of the outputs of the model after being spline
        % interpolated to match the sample size of the measurements
        InterpolatedOutputValues
        % numerical values of the states of the model after being spline
        % interpolated to match the sample size of the measurements
        InterpolatedStateValues
        % time vector for the control values
        ControlTime
        % original time stamp when experiment started
        ControlTimeStart
        % falcon.parest.Experiment/MeasurementTimeStart is a property.
        MeasurementTimeStart
        % numerical values of the control vector during the experiment
        ControlValues
        % time vector for the measurements
        MeasurementTime
        % time vector for the states
        StateTime
        % cell array of names of the controls in the experiment
        ControlNames
        % cell array of names of the measurements in the experiment
        MeasurementNames
        % cell array of names of the states in the experiment; is obtained
        % from the associated falcon.Phase object when setting it.
        StateNames
        % matrix of spline base functions to map the coarse outputs onto
        % the spline coefficients
        SplineMatrix_raw
        % matrix of spline base functions to map the spline coefficients to
        % the finer output grid
        SplineMatrix_fine
        % falcon.parest.plot.Experiment/PlotCfg_state is a property.
        PlotCfg_state
        % default plot properties for measurements, can be adjusted to
        % customize plot; if several signals are to be plotted in one
        % subplot, the array elements define the appearance of the
        % individual lines
        PlotCfg_meas
        % default plot properties for model, can be adjusted to customize
        % plot
        PlotCfg_model
        % default plot properties for controls, can be adjusted to
        % customize plot
        PlotCfg_control
        % falcon.parest.plot.Experiment/PlotCfg_procNoise is a property.
        PlotCfg_procNoise
        % default label properties, can be adjusted to customize labels
        Label_Cfg
        % cell array of label names of the controls in the experiment
        ControlLabels
        % falcon.parest.plot.Experiment/ProcessNoiseLabels is a property.
        ProcessNoiseLabels
        % falcon.parest.plot.Experiment/StateLabels is a property.
        StateLabels
        % cell array of label name of the time in the experiment
        TimeLabel
        % cell array of label names of the measurements in the experiment
        MeasurementLabels
        % factor to decrease the process noise sample time in order to
        % reduce number of variables, and make process noise better
        % identifiable. Process noise will be interpolated to fit
        % integration time steps; a factor < 1 is interpreted as percentage
        % of original samples to be kept. a factor > 1 is interpreted as
        % explicit number of samples to retain
        ProcNoiseSampleTimeFactor
        % size of process noise vector
        nW
        % falcon.parest.core.ProcNoiseExperiment/wfreeIdx is a property.
        wfreeIdx
        % falcon.parest.core.ProcNoiseExperiment/nWfree is a property.
        nWfree
        % sample size of the process noise, sampled down by
        % falcon.parest.Experiment.ProcNoiseSampleTimeFactor
        Nprocnoise
        % numericla values of the process noise of the model
        ProcessNoiseValues
        % falcon.parest.core.ProcNoiseExperiment/ProcessNoiseNames is a property.
        ProcessNoiseNames
        % falcon.parest.core.ProcNoiseExperiment/ProcessNoiseTime is a property.
        ProcessNoiseTime
        % numericla values of the process noise of the model, interpolated
        % to match the state grid
        StateInterpolatedProcessNoiseValues
        % numericla values of the process noise of the model, interpolated
        % to match the state grid
        OutputInterpolatedProcessNoiseValues
        % The process noise covariance matrix for a filter formulatoin
        Q
        % the initial state covariance matrix in a filter formulation
        P_xx_0
        % flag to indicate, if experiment has process noise input
        hasProcessNoise
        % falcon.parest.core.ProcNoiseExperiment/ProcessNoiseUpperBound is a property.
        ProcessNoiseUpperBound
        % falcon.parest.core.ProcNoiseExperiment/ProcessNoiseLowerBound is a property.
        ProcessNoiseLowerBound
        % falcon.parest.core.ProcNoiseExperiment/ProcessNoiseScaling is a property.
        ProcessNoiseScaling
        % the measurement noise covariance matrix in a filter formulatoin
        R
        % the outputs to consider in a filter formulation
        ConsideredOutputs
        % flag indicating if smoother is to be used
        SmootherFlag
        % the process noise feed-through matrix in a filter formulation
        E
        % the process noise of this experiment
        w
        % falcon.parest.core.ProcNoiseExperiment/ProcNoiseSplineMatrix_raw is a property.
        ProcNoiseSplineMatrix_raw
        % falcon.parest.core.ProcNoiseExperiment/ProcNoiseSplineMatrix_fine is a property.
        ProcNoiseSplineMatrix_fine
        % falcon.parest.core.hasSignalUncertainty/StateCovariance is a property.
        StateCovariance
        % falcon.parest.core.hasSignalUncertainty/OutputCovariance is a property.
        OutputCovariance
        % falcon.parest.core.hasSignalUncertainty/ProcesNoiseCovariance is a property.
        ProcesNoiseCovariance
        % falcon.parest.core.hasSignalUncertainty/StateStandardDeviation is a property.
        StateStandardDeviation
        % falcon.parest.core.hasSignalUncertainty/OutputStandardDeviation is a property.
        OutputStandardDeviation
        % falcon.parest.core.hasSignalUncertainty/ProcessNoiseStandardDeviation is a property.
        ProcessNoiseStandardDeviation
    end

    methods
        function ToFile(obj, FullFileName)
        % TOFILE store experiment data to file
        %  
        %  	<Syntax>
        %  	ToFile( obj, FullFileName )
        %  
        %  	<Description>
        %  	convert experiment data to struct and store this struct to a
        %  	file. the conversion to a struct makes the stored file
        %  	independent of the version of the class in use.
        %  
        %  	<Inputs>
        %  	>obj:
        %  	>FullFileName: full path and filename of the file to store
        %  	the experiment data in
        %  
        %  	<ChangeLog>
        %  	2017-01-29 - Ch. Goettlicher: initial design
        %  	2017-12-19 - Ch. Goettlicher: commenting (-->EMT completion)
        end

        function makeStateInitialGuess(obj, Method, varargin)
        % MAKESTATEINITIALGUESS generate initial guess for the state values
        %  
        %   <Syntax>
        %   obj.makeStateInitialGuess( 'simulate' )
        %   obj.makeStateInitialGuess( 'initialValues' )
        %   obj.makeStateInitialGuess( 'set', InitialGuessStates)
        %   obj.makeStateInitialGuess( 'filter' )
        %   obj.makeStateInitialGuess( 'filter', Q, R, P_xx_0)
        %   obj.makeStateInitialGuess( 'filter', Q, R, P_xx_0, ConsideredOutputs)
        %   obj.makeStateInitialGuess( 'filter', Q, R, P_xx_0, ConsideredOutputs, E)
        %   obj.makeStateInitialGuess( 'file', FileName)
        %  
        %  	<Description>
        %  	in order to apply the collocation estimation approach, an
        %  	initial guess for the state histories has to be provided.
        %  	Depending on the Method input to this function different
        %  	methods, different ways of creating the initial guess can be
        %  	applied.
        %  	> simulate: use the provided initial condition and control
        %  	inputs to do a forward simulation in order to obtain the
        %  	initial guess for the state values
        %  	> initialValues: fill the state history with the constant
        %  	initial condition
        %  	> set: set the provided state history in the second input
        %  	argument InitialGuessStates (which has to have the same
        %  	amount of samples as the measurements) and use it to compute
        %  	the corresponding output values via the model
        %  	> filter: use an extended kalman filter to reconstruct the
        %  	state and output histories. Either provide the necessary
        %  	additional information directly (Q, R, P_xx_0), or use the
        %  	falcon.parest.Experiment.setFilterData method to do so.
        %  	> file: load state and output values from file containing
        %  	experiments
        %  
        %  	<Inputs>
        %  	>obj: the Experiment object
        %  	>Method: the method to be used
        %  
        %  	<ChangeLog>
        %  	2017-01-29 - Ch. Goettlicher: initial design
        %  	2017-02-11 - Ch. Goettlicher: removed
        %                StateInitialGuessMethod as property, added
        %                filter possibility
        %  	2017-02-22 - Ch. Goettlicher: made possibility to load from
        %                file
        %  	2017-12-19 - Ch. Goettlicher: commenting (-->EMT completion)
        end

        function addConstraint(obj, varargin)
        % falcon.parest.Experiment/addConstraint is a function.
        %   addConstraint(obj, varargin)
        end

        function dy_dp = augmentJacEEM(obj, dy_dp)
        % AUGMENTJACEEM augment jacobian in EEM case
        %  
        %  	<Syntax>
        %  	[ dy_dp ] = augmentJacEEM( obj, dy_dp )
        %  
        %  	<Description>
        %  	adjust output sensitivities for EEM case by augmenting them
        %  	with jacobian of integration scheme in use (here,
        %  	trapezoidal). 
        %   This method is called at every iteration of the
        %   falcon.parest.solver, so it must not alter the output
        %   sensitivity in all other cases
        %  
        %  	<Inputs>
        %  	>obj: the falcon.parest.Experiment object
        %  	>dy_dp: the original output sensitivities
        %  
        %  	<Outputs>
        %  	>dy_dp: the (modified) output sensitivities
        %  
        %  	<ChangeLog>
        %  	2018-06-26 - Ch. Goettlicher: commenting
        end

        function dy_dp = get_dy_dp(obj)
        % GET_DY_DP obtain output sensitivity
        %  
        %  	<Syntax>
        %  	[ dy_dp ] = get_dy_dp( obj )
        %  
        %  	<Description>
        %  	get output sensitivity w.r.t. model parameters; output
        %  	jacobian of output euqation for EEM initial guess
        %  	improvement 
        %  
        %  	<Inputs>
        %  	>obj: the falcon.parest.Experiment object
        %  
        %  	<Outputs>
        %  	>dy_dp: output sensitivity of parameters
        %  
        %  	<ChangeLog>
        %  	2018-06-26 - Ch. Goettlicher: commenting
        end

        function dy_dx0 = get_dy_dx0(obj)
        % GET_DY_DX0 obtain output sensitivity
        %  
        %  	<Syntax>
        %  	[ dy_dx0 ] = get_dy_dx0( obj )
        %  
        %  	<Description>
        %  	get output sensitivity w.r.t. initial values; output zeros
        %  	only for EEM initial guess improvement (initial conditions
        %  	cannot be improved using this method)
        %  
        %  	<Inputs>
        %  	>obj: the falcon.parest.Experiment object
        %  
        %  	<Outputs>
        %  	>dy_dx0: output sensitivity of initial values
        %  
        %  	<ChangeLog>
        %  	2018-06-26 - Ch. Goettlicher: commenting
        end

        function z = getMeasurementValues(obj, varargin)
        % GETMEASUREMENTVALUES get measurement values
        %  
        %  	<Syntax>
        %   z = obj.getMeasurementValues(  )
        %   z = obj.getMeasurementValues( Indices )
        %   z = obj.getMeasurementValues('Name', Value)
        %  
        %  	<Description>
        %  	get the measurement values. these measurements are returned
        %  	as a cell array of doubles, where each element in the array
        %  	contains the measurement data for one experiment. Optionally,
        %  	an index vector can be passed, and only the relevant
        %  	measurements will be returned in the respective cells
        %  
        %  	<Inputs>
        %  	>obj: the Experiment object
        %  
        %  	<NameValue>
        %  	>Indices: logical or integer valued index vector to select
        %  	only certain measurements to be returned
        %  
        %  	<Outputs>
        %  	>z: the cell array containing the measurement values per
        %  	experiment
        %  
        %  	<ChangeLog>
        %  	2017-01-29 - Ch. Goettlicher: initial design
        %  	2017-12-19 - Ch. Goettlicher: commenting (-->EMT completion)
        %   2018-06-26 - Ch. Goettlicher: updated to EEM output
        %   capability
        end

        function y = getInterpolatedOutputValues(obj, varargin)
        % GETINTERPOLATEDOUTPUTVALUES get interpolated model output values
        %  
        %  	<Syntax>
        %  	y = obj.getInterpolatedOutputValues(  )
        %   y = obj.getInterpolatedOutputValues( Indices )
        %   y = obj.getInterpolatedOutputValues('Name', Value)
        %  
        %  	<Description>
        %  	get the model output values, interpolated to fit the
        %  	measurement grid. these outputs are returned as a cell array
        %  	of doubles, where each element in the array contains the
        %  	output data for one experiment. Optionally, an index vector
        %  	can be passed, and only the relevant output signals will be
        %  	returned in the respective cells
        %  
        %  	<Inputs>
        %  	>obj: the Experiment object
        %  
        %  	<NameValue>
        %  	>Indices: logical or integer valued index vector to select
        %  	only certain outputs to be returned
        %  
        %  	<Outputs>
        %  	>y: the cell array containing the interpolated output values
        %  	per experiment
        %  
        %  	<ChangeLog>
        %  	2017-01-29 - Ch. Goettlicher: initial design
        %  	2017-12-19 - Ch. Goettlicher: commenting (-->EMT completion)
        %   2018-06-26 - Ch. Goettlicher: adjusted to EEM improvement
        %   'mode'
        end

        function setMeasurementNames(obj, MeasurementNames)
        % SETMEASUREMENTNAMES set names of measurements
        %  
        %  	<Syntax>
        %  	setMeasurementNames( obj, MeasurementNames )
        %  
        %  	<Description>
        %  	set the names of the measurement signals by passing a cell
        %  	array of strings of proper size to this function. If this
        %  	method is called for an array of experiments, all of them
        %  	obtain the same measurement names
        %  
        %  	<Inputs>
        %  	>obj: the Experiment object
        %  	>MeasurementNames:
        %  	>ControlNames: cell array of strings containing the signal
        %  	names of the control inputs
        %  
        %  	<ChangeLog>
        %  	2017-01-29 - Ch. Goettlicher: initial design
        %  	2017-12-19 - Ch. Goettlicher: commenting (-->EMT completion)
        end

        function setControlNames(obj, ControlNames)
        % SETCONTROLNAMES set names of control inputs
        %  
        %  	<Syntax>
        %  	setControlNames( obj, ControlNames )
        %  
        %  	<Description>
        %  	set the names of the control signals by passing a cell array
        %  	of strings of proper size to this function. If this method is
        %  	called for an array of experiments, all of them obtain the
        %  	same control input names
        %  
        %  	<Inputs>
        %  	>obj:
        %  	>ControlNames: cell array of strings containing the signal
        %  	names of the control inputs
        %  
        %  	<ChangeLog>
        %  	2017-01-29 - Ch. Goettlicher: initial design
        %  	2017-12-19 - Ch. Goettlicher: commenting (-->EMT completion)
        end

        function s = ToStruct(obj)
        % TOSTRUCT store experiment information in struct
        %  
        %  	<Syntax>
        %  	[ s ] = ToStruct( obj )
        %  
        %  	<Description>
        %  	store all information of this experiment object in a struct.
        %  	Can be used to store the experiment data permanently, even if
        %  	the class definition changes (which would make storing the
        %  	complete object of a former version useless)
        %  
        %  	<Inputs>
        %  	>obj:
        %  
        %  	<Outputs>
        %  	>s: the experiment's information in a struct.
        %  
        %  	<ChangeLog>
        %  	2017-01-29 - Ch. Goettlicher: initial design
        %  	2017-01-29 - Ch. Goettlicher: store data un-sorted, i.e. in
        %                original order
        %  	2017-01-29 - Ch. Goettlicher: use ToStruct method of
        %                parameter for x0; store StateValues and
        %                OutputValues for warmstart
        %  	2017-01-29 - Ch. Goettlicher:
        %  	2017-12-19 - Ch. Goettlicher: commenting (-->EMT completion)
        end

        function obj = vertcat(varargin)
        % VERTCAT custom vertical concatenation method
        %  
        %  	<Syntax>
        %   obj = horzcat( obj1, ... objN )
        %  
        %   <Description>
        %   concatenates arrays of falcon.parest.Experiment objects.
        %   Makes sure, that the result is always a coloumn vector.
        %   Also issues a warning, if an experiment is used more than
        %   once.
        %  
        %   <Inputs>
        %  	>obj1, ... objN: arrays of falcon.parest.Experiment objects
        %  
        %  	<Outputs>
        %  	>obj: the concatenated object
        %  
        %  	<ChangeLog>
        %  	2017-02-17 - Ch. Goettlicher: initial design
        %  	2017-12-19 - Ch. Goettlicher: commenting (-->EMT completion)
        end

        function obj = horzcat(varargin)
        % HORZCAT custom horizontal concatenation method
        %  
        %   <Syntax>
        %   obj = horzcat( obj1, ... objN )
        %  
        %  	<Description>
        %  	concatenates arrays of falcon.parest.Experiment objects.
        %  	Makes sure, that the result is always a coloumn vector. Also
        %  	issues a warning, if an experiment is used more than once.
        %  
        %  	<Inputs>
        %   >obj1, ... objN: arrays of falcon.parest.Experiment objects
        %  
        %  	<Outputs>
        %  	>obj: the concatenated object
        %  
        %  	<ChangeLog>
        %  	2017-02-17 - Ch. Goettlicher: initial design
        %  	2017-12-19 - Ch. Goettlicher: commenting (-->EMT completion)
        end

        function obj = Experiment(ControlTime, ControlValues, Time, Measurements)
        % EXPERIMENT constructor for an experiment object
        %  
        %  	<Syntax>
        %  	[ obj ] = Experiment( obj, ControlValues, Time, Measurements )
        %  
        %  	<Description>
        %  	constructs an experiment obejct (closely related to the
        %  	underlying falcon.Phase object) from measured input/output
        %  	data.
        %  
        %  	<Inputs>
        %  	>ControlValues: numerical values of the measured controls
        %  	>Time: time vector of the measurements
        %  	>Measurements: numerical values of the measurements
        %  	>ControlTime: time vector of the measured controls
        %  
        %  	<Outputs>
        %  	>obj: the newly created object
        %  
        %  	<ChangeLog>
        %  	2017-01-29 - Ch. Goettlicher: initial design
        %  	2017-12-19 - Ch. Goettlicher: commenting (-->EMT completion)
        end

        function ret = eq(a, b)
        % EQ uality comparison for experiments
        %  
        %  	<Syntax>
        %   ret = eq(a, b)
        %   ret = a == b
        %  
        %  	<Description>
        %  	equality comparison for experiments. Numeric properties,
        %  	measurement and control names, as well as tags are compared
        %  	to check for equality
        %  
        %  	<Inputs>
        %  	>b: second array to compare
        %  	>a: first array to compare
        %  
        %  	<Outputs>
        %  	>ret: logical array to indicate equality
        %  
        %  	<ChangeLog>
        %  	2017-02-21 - Ch. Goettlicher: initial design
        %  	2017-11-04 - Ch. Goettlicher: exclude sub-structs in tags
        %                field from comparison
        %  	2017-12-19 - Ch. Goettlicher: commenting (-->EMT completion)
        end

        function obj = FromStruct(s)
        % FROMSTRUCT create experiment object from struct
        %  
        %  	<Syntax>
        %   obj = falcon.parest.Experiment.FromStruct(s)
        %  
        %  	<Description>
        %  	create a new experiment object from the data stored in a
        %  	struct
        %  
        %  	<Inputs>
        %  	>s: the struct containing the experiment data
        %  
        %  	<Outputs>
        %  	>obj: the newly created experiment object
        %  
        %  	<ChangeLog>
        %  	2017-01-29 - Ch. Goettlicher: initial design
        %  	2017-02-22 - Ch. Goettlicher: use fromStruct method of parameter for x0
        %  	2017-11-17 - Ch. Goettlicher: load state and output data, too.
        %  	2017-12-19 - Ch. Goettlicher: commenting (-->EMT completion)
        end

        function obj = FromFile(FullFileName)
        % FROMFILE load experiment from file
        %  
        %  	<Syntax>
        %  	[ obj ] = falcon.parest.Experiment.FromFile( FullFileName )
        %  
        %  	<Description>
        %  	create a new falcon.parest.Experiment object from a struct
        %  	stored in a file
        %  
        %  	<Inputs>
        %  	>FullFileName: full path and file name to the file
        %  	containing the experiment data
        %  
        %  	<Outputs>
        %  	>obj: the newly created experiment object
        %  
        %  	<ChangeLog>
        %  	2017-01-29 - Ch. Goettlicher: initial design
        %  	2017-12-19 - Ch. Goettlicher: commenting (-->EMT completion)
        end

        function setTimeLabel(obj, TimeLabel)
        % SETTIMELABELS set label name of time
        %  
        %   <Syntax>
        %   obj.setTimeLabel( TimeLabel )
        %  
        %   <Description>
        %   set the label of the time by passing a cell array of
        %   strings of proper size (1 by 1) to this function. If this
        %   method is called for an array of experiments, all of them
        %   obtain the same time label
        %  
        %   <Inputs>
        %   >TimeLabel: cell array of strings containing the label of
        %   the time
        %  
        %  	<ChangeLog>
        %   2017-12-12 - X. Fang: initial design
        %   2018-06-26 - Ch. Goettlicher: moved labels here
        end

        function setStateLabels(obj, StateLabels)
        % falcon.parest.plot.Experiment/setStateLabels is a function.
        %   setStateLabels(obj, StateLabels)
        end

        function setProcessNoiseLabels(obj, ProcessNoiseLabels)
        % falcon.parest.plot.Experiment/setProcessNoiseLabels is a function.
        %   setProcessNoiseLabels(obj, ProcessNoiseLabels)
        end

        function setMeasurementLabels(obj, MeasurementLabels)
        % SETMEASUREMENTLABELS set label names of measurements
        %  
        %   <Syntax>
        %   obj.setMeasurementLabels( MeasurementLabels )
        %  
        %   <Description>
        %   set the labels of the measurement signals by passing a cell
        %   array of strings of proper size to this function. If this
        %   method is called for an array of experiments, all of them
        %   obtain the same measurement labels
        %  
        %   <Inputs>
        %   >MeasurementLabels: cell array of strings containing the
        %   labels of the measurements
        %  
        %  	<ChangeLog>
        %   2017-12-12 - X. Fang: initial design
        %   2018-06-26 - Ch. Goettlicher: moved labels here
        end

        function setControlLabels(obj, ControlLabels)
        % SETCONTROLLABELS set label names of control inputs
        %  
        %   <Syntax>
        %   obj.setControlLabels( ControlLabels )
        %  
        %   <Description>
        %   set the names of the control signals by passing a cell
        %   array of strings of proper size to this function. If this
        %   method is called for an array of experiments, all of them
        %   obtain the same control input names
        %  
        %   <Inputs>
        %   >ControlLabels: cell array of strings containing the label
        %   of the control inputs
        %  
        %  	<ChangeLog>
        %   2017-12-12 - X. Fang: initial design
        %   2018-06-26 - Ch. Goettlicher: moved labels here
        end

        function updateStates(obj)
        % get y-limits for plots from all outputs and measurements
        end

        function updateProcessNoise(obj)
        % get y-limits for plots from all outputs and measurements
        end

        function updateModelOutput(obj)
        % UPDATEMODELOUTPUT update model output plots
        %  
        %  	<Syntax>
        %  	updateModelOutput( obj )
        %  
        %  	<Description>
        %  	updates the model output structs by adjusting the
        %  	corresponding .YData property of the plot objects. This is
        %  	faster than plotting the respective line anew and is mainly
        %  	used within the optimization iteration.
        %  
        %  	<Inputs>
        %  	>obj: the Experiment object
        %  
        %  	<ChangeLog>
        %  	2017-01-31 - Ch. Goettlicher: initial design
        %  	2017-12-19 - Ch. Goettlicher: commenting (-->EMT completion)
        end

        function plotUncertainties(obj, varargin)
        % falcon.parest.plot.Experiment/plotUncertainties is a function.
        %   plotUncertainties(obj, varargin)
        end

        function plotModelOutput(obj)
        % PLOTMODELOUTPUT plot model output
        %  
        %  	<Syntax>
        %  	plotModelOutput( obj )
        %  
        %  	<Description>
        %  	if a plot is associated with this experiment, plot Model
        %  	outputs, using the signals in
        %  	falcon.parest.plot.Experiment.OutputsToPlot and the plot
        %  	configuration of falcon.parest.plot.Experiment.PlotCfg_model.
        %  	the last model output plot will be updated during the
        %  	iteration, if the corresponding property
        %  	falcon.parest.Problem.CostFunction.UpdatePlots is set to a
        %  	value greater than 0
        %  
        %  	<Inputs>
        %  	>obj: the Experiment object
        %  
        %  	<ChangeLog>
        %  	2017-01-31 - Ch. Goettlicher: initial design
        %  	2017-12-19 - Ch. Goettlicher: commenting (-->EMT completion)
        %   2018-06-26 - Ch. Goettlicher: plot separation bars on top
        end

        function plotExperiment(obj, varargin)
        % plot this experiment
        %  
        %   <Syntax>
        %   obj.plotExperiment( )
        %   obj.plotExperiment( Outputs )
        %   obj.plotExperiment( Outputs, Controls, Tags )
        %   obj.plotExperiment( )
        %  
        %  	<Description>
        %  	plot the experiment data, i.e. measurements and controls.
        %  	Via the name-value pair input arguments, certain outputs
        %  	and controls can be selected for the plot, otherwise all
        %  	controls and measurements will be plotted. Also, certain
        %  	Tags can be chosen to be printed on top of the experiment
        %  	data. Per default none are shown.
        %  	The appearance of the plot can be configured via the fields
        %  	of the falcon.parest.plot.Experiment.PlotConfig struct If
        %  	the Outputs or Controls input is only a cell array of
        %  	strings with signal names, every signal is plotted in its
        %  	own subplot. Elements of this cell array of strings can be
        %  	replaced by cell array of strings, in which case the
        %  	respective signals will be plotted in the same subplot.
        %  	e.g. 'Controls', {{'xi_l_pos', 'xi_r_pos'}, 'zeta_pos'}
        %  	plots the signals xi_l_pos and xi_r_pos in one subplot,
        %  	wheras zeta_pos is plotted in its own subplot.
        %  
        %  	<Inputs>
        %  	>obj: the Experiment object
        %  
        %  	<NameValue>
        %  	>Outputs: cell array of output names, which are to be
        %  	included in the plot
        %  	>Controls: cell array of control names, which are to be
        %  	included in the plot
        %  
        %  	<ChangeLog>
        %  	2017-01-31 - Ch. Goettlicher: initial design
        %   2017-11-21 - Ch. Goettlicher: plot several lines together
        %  	2017-12-19 - Ch. Goettlicher: commenting (-->EMT completion)
        %   2018-06-26 - Ch. Goettlicher: include name-assert; adjusted
        %   label things
        end

        function hasPlotIdx = hasPlot(obj)
        % HASPLOT check of this experiment array is plotted
        %  
        %  	<Syntax>
        %  	[ hasPlotIdx ] = hasPlot( obj )
        %  
        %  	<Description>
        %  	return an array of indices, indicating if the respective
        %  	element in the experiment array is associated with any plots.
        %  	To do so, the size of the OutputFields and ControlFields
        %  	properties are checked
        %  
        %  	<Inputs>
        %  	>obj: the Experiment object
        %  
        %  	<Outputs>
        %  	>hasPlotIdx: returned array of indices of experiments, which
        %  	have plots associated with them.
        %  
        %  	<ChangeLog>
        %  	2017-01-31 - Ch. Goettlicher: initial design
        %  	2017-12-19 - Ch. Goettlicher: commenting (-->EMT completion)
        end

        function clearPlots(obj)
        % CLEARPLOTS clear all plots
        %  
        %  	<Syntax>
        %  	clearPlots( obj )
        %  
        %  	<Description>
        %  	clear output and control field properties
        %  
        %  	<Inputs>
        %  	>obj: the Experiment object
        %  
        %  	<ChangeLog>
        %  	2017-12-19 - Ch. Goettlicher: commenting (-->EMT completion)
        end

        function makeProcNoiseInitialGuess(obj, varargin)
        % MAKEPROCNOISEINITIALGUESS make initial guess for process noise
        %  
        %  	<Syntax>
        %  	makeProcNoiseInitialGuess( obj )
        %  
        %  	<Description>
        %  	make intial guess for process noise terms. Use initial
        %  	state guess to compute initial state derivatives, and
        %  	assign process noise such, that all integration defects are
        %  	initially at a minimum. This is done using an OLS solution.
        %   !!!!!!!!!! the process noise computede here may violate
        %   its boundaries! however, they will be enforced by the
        %   optimizer !!!!!!!!
        %  
        %  	<Inputs>
        %  	>obj: the falcon.parest.Experiment object
        %  
        %  	<ChangeLog>
        %  	2018-06-26 - Ch. Goettlicher: commenting
        end

        function setFilterData(obj, varargin)
        % SETFILTERDATA set the additional data, necessary for the filter
        %  
        %   <Syntax>
        %   obj.setFilterData( R, Q, P_xx_0)
        %   obj.setFilterData( R, Q, P_xx_0, ConsideredOutputs)
        %   obj.setFilterData( R, Q, P_xx_0, ConsideredOutputs, E)
        %  
        %   <Description>
        %  	store the filter data and afterwards call the checkFilterData
        %  	method in order to verify, that all dimensions etc. are
        %  	proper
        %  
        %  	<Inputs>
        %  	>obj: the Experiment object
        %  	>Q: process noise covariance matrix
        %  	>R: measurement noise covariance matrix
        %  	>P_xx_0: initial state covariance matrix
        %  
        %  	<NameValue>
        %  	>ConsideredOutputs: logical array indicating, which outputs
        %  	to use. default is all outputs. can also be string
        %  	'Outputs', then the model's output's 'isConsidered'
        %  	configuration is used.
        %  	>E: process noise feedthrough matrix
        %  
        %  	<ChangeLog>
        %  	2017-02-11 - Ch. Goettlicher: initial design
        %  	2017-12-19 - Ch. Goettlicher: commenting (-->EMT completion)
        end

    end

end