classdef ExperimentFilter < falcon.core.Handle
    % FALCON.PAREST.EXPERIMENTFILTER filtering functions for experiments
    %  
    %  	<Description>
    %  	this class can be used to select certain experiments from an array of
    %  	experiments according to their Tags. To do so, certain comparisons can
    %  	be used to filter experiments. For example, to get all experiments,
    %  	whose 'Direction' Tag is the string 'Long' use
    %       Filter = falcon.parest.ExperimentFilter();
    %       Filter.setFilter('Direction', 'substr', 'Long');
    %       experiments_Long = Filter.filter( experiments );
    %  
    %   To obtain all experiments, whose 'TrimSpeed' Tag is larger than 20 use
    %       Filter = falcon.parest.ExperimentFilter();
    %       Filter.setFilter('TrimSpeed', '>=', 20);
    %       experiments_Long = Filter.filter( experiments );
    %  
    %   combining filters results in a logical 'and' combination of the
    %   comparison results
    %  
    %  	<ChangeLog>
    %  	2017-12-19 - Ch. Goettlicher: commenting (-->EMT completion)

    % -------------------------------------------------------------------------
    %                                FALCON.m
    % Copyright (c) 2014-2019 Institute of Flight System Dynamics, TUM, Munich
    % Matthias Bittner, Matthias Rieck, Maximilian Richter,
    % Benedikt Grueter, Johannes Diepolder, Florian Schwaiger,
    % Patrick Piprek, and Florian Holzapfel
    % Downloading, using, copying, or modifying FALCON.m code constitutes an
    % agreement to ALL of the terms of the FALCON.m EULA.
    % -------------------------------------------------------------------------

    properties
        % list of tags to be filtered
        Tags
        % list of constraints
        Constraint
        % list of comparitors
        Comparator
        % number of comparisons to be done
        NComparisons
        % all possible comparators
        PossibleComparators
    end

    methods
        function ExperimentArray_out = filter(obj, ExperimentArray)
        % FILTER apply filter to array of experiments
        %  
        %  	<Syntax>
        %  	[ ExperimentArray_out ] = filter( obj, ExperimentArray )
        %  
        %  	<Description>
        %  	apply filter rules defined by
        %  	falcon.parest.ExperimentFilter.setFilter to array of
        %  	experiments
        %  
        %  	<Inputs>
        %  	>obj: the ExperimentFilter object
        %  	>ExperimentArray: array of experiments to be filtered
        %  
        %  	<Outputs>
        %  	>ExperimentArray_out: filtered experiment array
        %  
        %  	<ChangeLog>
        %  	2017-01-26 - Ch. Goettlicher: initial design
        %  	2017-12-19 - Ch. Goettlicher: commenting (-->EMT completion)
        end

        function setFilter(obj, TagName, Constraint, Comparator)
        % SETFILTER set filter rules
        %  
        %  	<Syntax>
        %  	setFilter( obj, TagName, Constraint, Comparator )
        %  
        %  	<Description>
        %  	set one filter rule, i.e. tag to be filtered, constraint to
        %  	be checked and comparator to be applied.
        %  	multiple filters can be set by calling this method several
        %  	times.
        %  
        %  	<Inputs>
        %  	>obj: the ExperimentFilter object
        %  	>TagName: Name of the tag to be filtered
        %  	>Constraint: constraint to be checked (e.g. value, string,
        %  	flag etc.)
        %  	>Comparator: comparator to be used (e.g. is equal, larger
        %  	than, smaller than, contains, etc.)
        %  
        %  	<ChangeLog>
        %  	2017-01-26 - Ch. Goettlicher: initial design
        %  	2017-12-19 - Ch. Goettlicher: commenting (-->EMT completion)
        end

        function obj = ExperimentFilter()
        % EXPERIMENTFILTER constructor
        %  
        %  	<Syntax>
        %  	[ obj ] = ExperimentFilter( obj )
        %  
        %  	<Description>
        %  	constructs experiment filter object
        %  
        %  	<Outputs>
        %  	>obj: the ExperimentFilter object
        %  
        %  	<ChangeLog>
        %   2017-01-26 - Ch. Goettlicher: initial design
        %  	2017-12-19 - Ch. Goettlicher: commenting (-->EMT completion)
        end

    end

end