classdef Problem < falcon.parest.core.Problem
    % FALCON.PAREST.PROBLEM estimation using deterministic model
    %  
    %  	<Description>
    %  	child class of generic Problem definition for the application of
    %  	deterministic models.
    %  
    %  	<ChangeLog>
    %  	2017-12-19 - Ch. Goettlicher: commenting (-->EMT completion)

    % -------------------------------------------------------------------------
    %                                FALCON.m
    % Copyright (c) 2014-2019 Institute of Flight System Dynamics, TUM, Munich
    % Matthias Bittner, Matthias Rieck, Maximilian Richter,
    % Benedikt Grueter, Johannes Diepolder, Florian Schwaiger,
    % Patrick Piprek, and Florian Holzapfel
    % Downloading, using, copying, or modifying FALCON.m code constitutes an
    % agreement to ALL of the terms of the FALCON.m EULA.
    % -------------------------------------------------------------------------

    properties
        % array of falcon.parest.Experiment objects containing different
        % input / output data-sets for different experiments
        Experiments
        % handle of model function; has to be created using the simulation
        % model builder falcon.parest.SimulationModelBuilder
        ModelHandle
        % Current Cost Function object. Different choices are possible,
        % e.g. maximum likelihood, modified maximum likelihood,
        % least-squares. use method falcon.parest.Problem.setCostFunction
        % to explicitly set it.
        % Default is falcon.parest.costfunction.MaximumLikelihood
        CostFunction
        % vector of falcon.parest.State objects, defining the system's
        % states;
        States
        % vector of falcon.parest.Control objects, defining the system's
        % controls; with falcon.parest.Control objects, their isFixed
        % property is always set to true!
        Controls
        % vector of falcon.parest.Output objects, defining the system's
        % outputs; via their isConsidered property it can be chosen, if the
        % respective output is to be considered in the estimation
        Outputs
        % vector of falcon.parest.Parameter objects, defining the system's
        % model parameters; additionally, falcon.parest.Parameter objects
        % are created for the initial conditions of the experiments
        ModelParameters
        % size of parameter vector
        nP
        % size of state vector
        nX
        % size of output vector
        nY
        % size of input vector
        nU
        % number of experiments
        nExperiment
        % index of outputs to be considered in the estimation process
        RelevantOutputsIdx
        % Array keeping all falcon.core.Phases of this problem.
        Phases
        % Array keeping all falcon.Parameters of this problem.
        Parameters
        % Array keeping all falcon.core.PointFunction objects of this problem.
        PointConstraintFunctions
        % Array keeping all falcon.core.PointFunction objects used as Mayer
        % cost functions of this problem.
        MayerCostFunctions
        % Determines if the optimization problem has already been
        % baked. If true, the problem can no longer be altered. Use
        % falcon.Problem.UnBake to make it editable again.
        isCrunchy
        % The falcon.discretization.DiscretizationMethod method used to
        % discretize this optimal control problem. Use
        % falcon.Problem.setDiscretizationMethod to set this property.
        % Available discretization methods can be found in
        % falcon.discretization.'name'. (default =
        % falcon.discretization.Trapezoidal)
        DiscretizationMethod
        % Offset for the sum of all cost functions
        CostOffset
        % The numeric scaling factor for the sum of all cost functions.
        CostScaling
        % Flag that determines if the analytic Hessian (2nd Derivative)
        % of the problem is calculated (ipopt only).
        % The Hessian mode is invoked in the falcon.Problem
        % constructor.
        UseHessian
        % Flag that determines use of drawnow to allow for optimization
        % abort in OptiFunc.
        DisableAbort
        % Flag to determine whether the problem is already solved
        isSolved
        % Use simulation as initial guess for state
        doSimInitGuess
        % MajorIterLimit for shortcut problem.Solve
        MajorIterLimit
        % MajorFeasTol for shortcut problem.Solve
        MajorFeasTol
        % MajorOptTol for shortcut problem.Solve
        MajorOptTol
        % Get the sparsity pattern of the gradient matrix as a sparse matrix
        gSparsity
        % Get the sparsity pattern of the Hessian matrix as a sparse matrix
        HSparsity
        % extract and concatenate all state values
        StateValues
        % extract and concatenate all state names
        StateNames
        % extract and concatenate all control values
        ControlValues
        % extract and concatenate all control names
        ControlNames
        % extract and concatenate all output values
        OutputValues
        % extract and concatenate all output names
        OutputNames
        % extract and concatenate all state_dot values
        StateDotValues
        % extract and concatenate all output names
        StateDotNames
        % extract and concatenate all costates values
        CostateValues
        % extract and concatenate all post-processing values
        PostProcessedValues
        % extract and concatenate all post-processing names
        PostProcessedNames
        % extract real time of problem
        RealTime
        % Name of object.
        Name
        % falcon.parest.core.UncertaintyComputation/SolverEvolution is a property.
        SolverEvolution
        % falcon.parest.core.ConstraintInformation/Solver is a property.
        Solver
        % falcon.parest.core.ConstraintInformation/z_ConstrActiveUB is a property.
        z_ConstrActiveUB
        % falcon.parest.core.ConstraintInformation/z_ConstrActiveLB is a property.
        z_ConstrActiveLB
        % falcon.parest.core.ConstraintInformation/z_ConstrActiveEQ is a property.
        z_ConstrActiveEQ
        % falcon.parest.core.ConstraintInformation/F_ConstrActiveUB is a property.
        F_ConstrActiveUB
        % falcon.parest.core.ConstraintInformation/F_ConstrActiveLB is a property.
        F_ConstrActiveLB
        % falcon.parest.core.ConstraintInformation/F_ConstrActiveEQ is a property.
        F_ConstrActiveEQ
        % falcon.parest.core.ConstraintInformation/z_ConstrActive is a property.
        z_ConstrActive
        % falcon.parest.core.ConstraintInformation/F_ConstrActive is a property.
        F_ConstrActive
        % falcon.parest.core.ConstraintInformation/z_Free is a property.
        z_Free
        % falcon.parest.core.ConstraintInformation/F_Free is a property.
        F_Free
        % falcon.parest.core.ConstraintInformation/z_ConstrActiveidx is a property.
        z_ConstrActiveidx
        % falcon.parest.core.ConstraintInformation/F_ConstrActiveidx is a property.
        F_ConstrActiveidx
        % falcon.parest.core.ConstraintInformation/z_Freeidx is a property.
        z_Freeidx
        % falcon.parest.core.ConstraintInformation/F_Freeidx is a property.
        F_Freeidx
        % falcon.parest.core.ConstraintInformation/z_ConstrActiveUBidx is a property.
        z_ConstrActiveUBidx
        % falcon.parest.core.ConstraintInformation/z_ConstrActiveLBidx is a property.
        z_ConstrActiveLBidx
        % falcon.parest.core.ConstraintInformation/z_ConstrActiveEQidx is a property.
        z_ConstrActiveEQidx
        % falcon.parest.core.ConstraintInformation/F_ConstrActiveUBidx is a property.
        F_ConstrActiveUBidx
        % falcon.parest.core.ConstraintInformation/F_ConstrActiveLBidx is a property.
        F_ConstrActiveLBidx
        % falcon.parest.core.ConstraintInformation/F_ConstrActiveEQidx is a property.
        F_ConstrActiveEQidx
    end

    methods
        function obj = Problem(name, varargin)
        % PROBLEM constructor
        %  
        %  	<Syntax>
        %  	[ obj ] = Problem( name, varargin )
        %  
        %  	<Description>
        %  	Detailed explanation goes here
        %  
        %  	<Inputs>
        %  	>name: Problem name
        %  	>varargin: input arguments to be passed to superclass
        %  	constructor
        %  
        %  	<Outputs>
        %  	>obj: the Problem object
        %  
        %  	<ChangeLog>
        %  	2017-12-19 - Ch. Goettlicher: commenting (-->EMT completion)
        end

        function addNewPointConstraint(obj, FunctionHandle, Constraints)
        % superclass method not to be used in falcon.parest.Problem
        %             error('falcon:parest:ForbiddenFunction', 'function addNewPointConstraint may not be used with falcon.parest.Problem');
        end

        function Bake(obj)
        % BAKE the problem
        %  
        %  	<Syntax>
        %  	Bake( obj )
        %  
        %  	<Description>
        %  	prepare the problem for optimization; This method only has to
        %  	be called when ipopt is used to do the estimation.
        %  
        %  	<Inputs>
        %  	>obj: the Problem object
        %  
        %  	<ChangeLog>
        %   2017-01-24 - Ch. Goettlicher: initial design
        %  	2017-12-19 - Ch. Goettlicher: commenting (-->EMT completion)
        end

        function improveInitialModelParameters(obj, method, varargin)
        % IMPROVEINITIALMODELPARAMETERS improve initial guess for p
        %  
        %  	<Syntax>
        %  	improveInitialModelParameters( obj, method )
        %  	improveInitialModelParameters( obj, method, Name, Value, ...)
        %  
        %  	<Description>
        %  	automatically improve initial guess for model parameters,
        %  	based on the initial state guess. Two methods are
        %  	implemented:
        %       * useInitialStateGuess: use initial state guess to
        %       compute output sensitivities instead of propagated
        %       states; do parameter updates based on these output
        %       sensitivities, which are hopefully closer to the final
        %       values, compared to those that are based on the
        %       propagated states using bad parameter values.
        %       * EEM: equation error approach to initial guess
        %       improvement. consider initial state guess as given, and
        %       minimize integration defects and output residuals
        %       simultaneously by varying the model parameters.
        %  
        %   Per default, the maximum likelihood cost function is used
        %   (i.e. estimating the residual covariance and minimizing its
        %   determinant). Via an optional input argument, the use of
        %   the cost function of the problem can be enforced.
        %  
        %   the custom maximum likelihood solver falcon.parest.solver
        %   together with the LevenbergMarquardt step algorithm is used
        %   for the solution of above problem.
        %  
        %  	<Inputs>
        %  	>obj: the Model object
        %   >method: the method to be used (useInitialStateGuess or
        %   EEM)
        %  
        %   <NameValue>
        %   >ForceParentCost: logical to enforce the use of the
        %   problem's cost function instead of the maximum likelihood
        %   cost
        %   >MajorIterLimit: iteration limit. Convergence should be
        %   achieved quickly, so small numbers may be used.
        %   >PrintLevel: print level for solver, default 0, i.e. no
        %   output
        %  
        %  	<ChangeLog>
        %  	2018-06-26 - Ch. Goettlicher: initial design
        end

        function addNewExperiment(obj, Experiments, varargin)
        % ADDNEWEXPERIMENT Add a new Experiment to the Problem.
        %  
        %  	<Syntax>
        %  	addNewExperiment( obj, Experiments )
        %  
        %  	<Description>
        %  	add the experiments to the problem, which are to be
        %  	considered in the estimation process.
        %  
        %  	<Inputs>
        %  	>obj: the Problem object
        %  	>Experiments: vector of falcon.parest.Experiment objects
        %  	containing the input/output data as well as information about
        %  	the initial conditions for the respective experiments to be
        %  	used in the estimation process.
        %  
        %  	<ChangeLog>
        %  	2017-01-24 - Ch. Goettlicher: initial design
        %  	2017-12-19 - Ch. Goettlicher: commenting (-->EMT completion)
        end

        function addModel(obj, varargin)
        % ADDMODEL add a model to the estimation problem
        %  
        %  	<Syntax>
        %   obj.addModel(ModelHandle, parameter)
        %   obj.addModel(ModelHandle, parameter, state)
        %   obj.addModel(ModelHandle, parameter, state, control)
        %   obj.addModel(ModelHandle, parameter, state, control, output)
        %   obj = constructor(..., 'Name', Value)
        %  
        %  	<Description>
        %  	add a model handle and the corresponding parameters (as
        %  	falcon.parest.Parameter objects) to the problem. Optionally,
        %  	states, controls and outputs of respective falcon.parest-type
        %  	can be added, too.
        %  
        %  	<Inputs>
        %  	>obj: the Problem object
        %  	>ModelHandle: function handle of a model function generated
        %  	using the falcon.parest.SimulationModelBuilder
        %  	>parameter: vector of falcon.parest.Parameter objects to be
        %  	determined in this estimation problem.
        %  
        %  	<NameValue>
        %  	>state: state vector corresponding to the model. If omitted,
        %  	default falcon.parest.State objects are created based on the
        %  	model information.
        %  	>control: control vector corresponding to the model. If
        %  	omitted, default falcon.parest.Control objects are created
        %  	based on the model information.
        %  	>output: output vector corresponding to the model. If
        %  	omitted, default falcon.parest.Output objects are created
        %  	based on the model information.
        %  
        %  	<ChangeLog>
        %  	2017-01-24 - Ch. Goettlicher: initial design
        %  	2017-12-19 - Ch. Goettlicher: commenting (-->EMT completion)
        end

        function setCostFunction(obj, CostFunction)
        % SETCOSTFUNCTION set cost function object for this problem
        %  
        %  	<Syntax>
        %  	setCostFunction( obj, CostFunction )
        %  
        %  	<Description>
        %  	Different implementations of cost functions can be added to
        %  	the problem. They have to be of class
        %  	falcon.parest.costfunction.CostFunction. See the folder
        %  	+falcon/+parest/+costfunction for all possibilities.
        %  	If no cost function is explicitly set,
        %  	falcon.parest.costfunction.MaximumLikelihood will be used.
        %  
        %  	<Inputs>
        %  	>obj: the Problem object
        %  	>CostFunction: Cost function object to be used.
        %  
        %  	<ChangeLog>
        %  	2017-01-24 - Ch. Goettlicher: initial design
        %  	2017-12-19 - Ch. Goettlicher: commenting (-->EMT completion)
        end

        function OVC = checkSuppliedVsModelData(Supplied, Info)
        % CHECKSUPPLIEDVSMODELDATA check supplied and model list of
        %   OVC's and store them correspondingly
        %  
        %  	<Syntax>
        %  	[ OVC ] = checkSuppliedVsModelData( Supplied, Info )
        %  
        %  	<Description>
        %  	check, that the length of the supplied and model OVC have
        %  	the same length. Then sort the supplied list in the same
        %  	order as in the model. Check, that all of the model's OVCs
        %  	are present.
        %  
        %  	<Inputs>
        %  	>Supplied: list of supplied OVCs from the input parser
        %  	>Info: Model info
        %  
        %  	<Outputs>
        %  	>OVC: optimization value container, where the list is to be
        %  	stored (parameters, states, controls, outputs)
        %  
        %  	<ChangeLog>
        %  	2017-12-19 - Ch. Goettlicher: commenting (-->EMT completion)
        end

        function obj = setUseHessian(obj, useHessian)
        % falcon.Problem/setUseHessian is a function.
        %   obj = setUseHessian(obj, useHessian)
        end

        function [parameters, locations] = getParametersByName(self, names)
        % Get problem parameters by name.
        %  
        % <Syntax>
        % [parameters, locations] = problem.getParametersByName(names)
        %  
        % <Description>
        % Returns the parameters corresponding to the given names.  If a
        % parameter is not found, an exception is thrown.  If a name is
        % ambiguous, the first match is returned.
        %  
        % <Inputs>
        % > names: cellstr
        % > parameters: falcon.Parameter array
        % > locations: index array referring to problem.Parameters
        end

        function showCostFunctionValues(obj)
        % Extracts the values of the Mayer cost functions and prints them to the
        % console.
        %  
        % <Syntax>
        % obj.showCostFunctionValues()
        %  
        % <Description>
        % Show all Mayer cost functions in the console.
        %  
        % <Keywords>
        % Debugging!Problem!Cost Values
        end

        function PlotGUI(obj)
        % Opens the graphical user interface for plotting of the current problem.
        %  
        % <Syntax>
        % obj.PlotGUI();
        %  
        % <Description>
        % Creates a new instance of the plot GUI using the current problem. In the
        % plot GUI, the user can select all available time histories in the problem
        % and create plots from them.
        %  
        % <Keywords>
        % Problem!GUI
        end

        function setMajorIterLimit(obj, MajorIterLimitVal)
        % Set the major iteration limit for the problem.Solve
        % method.
        %  
        % <Syntax>
        % obj.setMajorIterLimit(MajorIterLimitVal)
        %  
        % <Description>
        % Set the major iteration limit for the method
        % problem.Solve. Value will be assigned there to the
        % automatically created solver object.
        %  
        % <NameValue>
        % >MajorIterLimitVal: Numeric value of the limit.
        %  
        % <Keywords>
        % Solver!Limit!Iteration
        end

        function setMajorOptTol(obj, MajorOptTolVal)
        % Set the major optimality tolerance for the problem.Solve
        % method.
        %  
        % <Syntax>
        % obj.setMajorOptTol(MajorOptTolVal)
        %  
        % <Description>
        % Set the major optimality tolerance for the method
        % problem.Solve. Value will be assigned there to the
        % automatically created solver object.
        %  
        % <NameValue>
        % >MajorOptTolVal: Numeric value of the tolerance.
        %  
        % <Keywords>
        % Solver!Tolerance!Optimality
        end

        function setMajorFeasTol(obj, MajorFeasTolVal)
        % Set the major feasibility tolerance for the problem.Solve
        % method.
        %  
        % <Syntax>
        % obj.setMajorFeasTol(MajorFeasTolVal)
        %  
        % <Description>
        % Set the major feasibility tolerance for the method
        % problem.Solve. Value will be assigned there to the
        % automatically created solver object.
        %  
        % <NameValue>
        % >MajorFeasTolVal: Numeric value of the tolerance.
        %  
        % <Keywords>
        % Solver!Tolerance!Feasibility
        end

        function [z_opt, F_opt, status, lambda, mu, zl, zu, solver] = Solve(obj, varargin)
        % Solve the problem.
        %  
        % <Syntax>
        % [z_opt, F_opt, status, lambda, mu, zl, zu] = obj.Solve()
        % [z_opt, F_opt, status, lambda, mu, zl, zu] = obj.Solve(Solver)
        %  
        % <Description>
        % Solve the problem numerically either using the default solver and the
        % default settings or using the given numeric solver.
        %  
        % <Inputs>
        % Solver: The falcon.solver used to solve the problem. (default: IPOPT)
        %  
        % <Keywords>
        % Problem!Solve; Solver!Solve
        end

        function UnBake(obj)
        % UnBake the problem. Make it editable again.
        %  
        % <Syntax>
        % obj.UnBake()
        %  
        % <Description>
        % Make a problem editable again, that was already baked. This is especially
        % usefull when solving similar problems again and again.
        %  
        % <Keywords>
        % Problem!UnBake
        end

        function [states, outputs, simTime, statesDot] = Simulate(obj, varargin)
        % Integrate the optimized trajectory using standard matlab
        % solvers. The result will be a cell array containing all the
        % simulated states for the phases.
        %  
        % <Syntax>
        % [states,outputs,simTime,statesDot] = obj.Simulate()
        % [states,outputs,simTime,statesDot] = obj.Simulate(init_states)
        % [states,outputs,simTime,statesDot] = obj.Simulate(init_states,control_history)
        % [states,outputs,simTime,statesDot] = obj.Simulate(init_states,control_history,ode_solver)
        % [states,outputs,simTime,statesDot] = obj.Simulate(init_states,control_history,ode_solver,ode_options)
        % [states,outputs,simTime,statesDot] = obj.Simulate(init_states,control_history,ode_solver,ode_options,UseRealTime)
        % [states,outputs,simTime,statesDot] = obj.Simulate(init_states,control_history,ode_solver,ode_options,UseRealTime,SplitIntervals)
        % [states,outputs,simTime,statesDot] = obj.Simulate(init_states,control_history,ode_solver,ode_options,UseRealTime,SplitIntervals,UseODETimeStep)
        % [states,outputs,simTime,statesDot] = obj.Simulate(init_states,control_history,ode_solver,ode_options,UseRealTime,SplitIntervals,UseODETimeStep,UseBackwardInt)
        % [states,outputs,simTime,statesDot] = obj.Simulate(init_states,control_history,ode_solver,ode_options,UseRealTime,SplitIntervals,UseODETimeStep,UseBackwardInt,DoVisualization)
        % [states,outputs,simTime,statesDot] = obj.Simulate(init_states,control_history,ode_solver,ode_options,UseRealTime,SplitIntervals,UseODETimeStep,UseBackwardInt,DoVisualization,DoSensitivity)
        %  
        % <Description>
        % Simulate the system with controls and initial state of the optimization.
        %  
        % <NameValue>
        % > init_states: The initial state vector for the split
        % intervals. If backward integration is used the final values.
        % Default are values from the current state grid.
        % > control_history: The control history to be used for the
        % simulation. Default is the interpolated control grid.
        % > ode_solver: Solver type that should be used for the
        % integration (default: ode45). Specified as a string.
        % > ode_options: Specify an ode options struct used for the ode
        % solver. As default the standard ode settings are used.
        % > UseRealTime: Use the real, i.e. physical, time for the
        % simulation instead of the non-dimensional time tau (default:
        % true).
        % > SplitIntervals: Number of split intervals, i.e. intervals
        % that split the integration domain. This generally makes the
        % integration more stable.
        % > UseODETimeStep: Flag that specifies whether to use the
        % internal ode time step for the measurements or the time steps
        % from the optimization. Result might be helpful to determine
        % the required number of collocation steps (default: false).
        % > UseBackwardInt: Flag that specifies whether to use the
        % backward or forward integration in time (default: false).
        % > DoVisualization: Flag to visualize the results (default:
        % false)
        %  
        % <Outputs>
        % > states: Simulated states.
        % > outputs: Simulated outputs.
        % > simTime: The time grid the integration is carried out.
        % > statesDot: The state derivatives.
        %  
        % <Keywords>
        % Problem!Simulation
        end

        function FastBake(obj, varargin)
        % Fast bake the problem, i.e., only change initial guesses and
        % boundary conditions.
        %  
        % <Syntax>
        % obj.FastBake()
        %  
        % <Description>
        % This method provides a fast bake method after an initial bake
        % has already been conducted. Within this function no new
        % indices are calculated but only the optimization parameter
        % vector is adapted with regard to state and control guess as
        % well as initial and final boundaries. No problem changes can
        % be done in this function!
        %  
        % <NameValue>
        % > initGuessCtrlsCell: A cell containing the initial guess for
        % the control of each phase. If a phase should not be changed
        % leave the cell empty.
        % > initGuessStateCell: A cell containing the initial guess for
        % the state of each phase.
        % > InitialBoundariesCell: A cell containing the initial
        % boundary condition of each phase.
        % > InitialBoundariesCell: A cell containing the final
        % boundary condition of each phase.
        %  
        % > UpdateInitialGuess: A logical to either overwrite (true) or
        % keep (false) the initial guess
        % <Keywords>
        % gPC Problem!Fast Bake
        end

        function setSimInitGuess(obj, SimInitGuessFlag)
        % Set the flag that determines whether to use a simulation for
        % the initial guess of the states within each phase or not.
        %  
        % <Syntax>
        % obj.setSimInitGuess(SimInitGuessFlag)
        %  
        % <Description>
        % Sets the boolean value of the simulation flag for the initial
        % guess.
        %  
        % <Inputs>
        % > SimInitGuessFlag: Boolean determining whether to use a
        % simulation for the initial guess (true) or not (false).
        % (defaul: false)
        %  
        % <Keywords>
        % Problem!Simulation!Flag
        end

        function setCostOffset(self, offset)
        % falcon.Problem/setCostOffset is a function.
        %   setCostOffset(self, offset)
        end

        function setDiscretizationMethod(obj, DiscretizationMethod)
        % Set the discretization method of this problem.
        %  
        % <Syntax>
        % obj.setDiscretizationMethod(DiscretizationMethod)
        %  
        % <Description>
        % Sets the given discretization method as the discretization method used to
        % solve this optimal control problem.
        %  
        % <Inputs>
        % > DiscretizationMethod: A handle to a child class of
        % falcon.discretization.DiscretizationMethod containing all the functions
        % required to discretize an optimal control problem before solving it. The
        % usable classes shipped with falcon can be found in falcon.discretization.
        %  
        % <Keywords>
        % Problem!Discretization Method; Solver!Discretization Method;
        end

        function [cost, pointFunction] = addNewQuadraticPointCost(self, variables, varargin)
        % Minimize (default) or maximize a quadratic form.
        %  
        % <Syntax>
        % problem.addNewQuadraticPointCost(variables)
        % problem.addNewQuadraticPointCost(variables, Phase)
        % problem.addNewQuadraticPointCost(variables, Phase, NormalizedTime)
        % problem.addNewQuadraticPointCost(..., 'Name', Value)
        %  
        % <Description>
        % Add a quadratic point cost function of the form
        %   sum[i] ( (v[i] - v0[i])' * W[i] * (v[i] - v0[i]) )
        % with a vector of variables v (including States, Controls,
        % Parameters and Outputs in arbitrary order), offset vector v0 and
        % weight matrix W; the subscript i represents a sample index.
        %  
        % The function accepts inputs from a single phase only.  However,
        % multiple samples from this phase may be used.
        %  
        % Note: If the function is applied to multiple samples and v
        % includes parameters, these are replicated to all samples.
        % For example, given v = [p] with a parameter p, constant weights
        % W[i] = W and offsets v0[i] = v0 = [p0] and N samples, the result
        % is N * (p - p0)' * W * (p - p0).
        end

        function [cost, pointFunction] = addNewLinearPointCost(self, variables, varargin)
        % Minimize (default) or maximize given variables.
        %  
        % <Syntax>
        % problem.addNewLinearPointCost(variables)
        % problem.addNewLinearPointCost(variables, Phase)
        % problem.addNewLinearPointCost(variables, Phase, NormalizedTime)
        % problem.addNewLinearPointCost(..., 'Name', Value)
        %  
        % <Description>
        % Add a linear point cost function of the form
        %   sum[i] ( w[i]' * (v[i] - v0[i]) )
        % with a vector of variables v (including States, Controls,
        % Parameters and Outputs in arbitrary order), offset vector v0 and
        % weight vector w; the subscript i represents a sample index.
        %  
        % Note: If the function is applied to multiple samples and v
        % includes parameters, these are replicated to all samples.
        % For example, given v = [p] with a parameter p, constant weights
        % w[i] = w and offsets v0[i] = v0 = [p0] and N samples, the result
        % is N * w * (p - p0).
        end

        function MayerFunction = addNewMayerCost(obj, FunctionHandle, varargin)
        % Add a new Mayer cost function to the Problem.
        %  
        % <Syntax>
        % obj.addNewMayerCost(FunctionHandle)
        % obj.addNewMayerCost(FunctionHandle, Cost)
        % obj.addNewMayerCost(FunctionHandle, Cost, Phase1, NormalizedTime1)
        % obj.addNewMayerCost(FunctionHandle, Cost, Phase1, NormalizedTime1, Phase2, NormalizedTime2)
        % obj.addNewMayerCost(FunctionHandle, Cost, ..., PhaseN, NormalizedTimeN)
        %  
        % <Description>
        % Creates a new PointFunction and adds it to the list of Mayer cost functions
        % of the problem.
        %  
        % <Inputs>
        % > FunctionHandle: The function handle to the function used to calculate
        % this path constraint. For more information on the function handle see
        % falcon.PointFunction or falcon.PointConstraintBuilder. If not specified via a
        % point constraint builder file, the function handle must
        % fulfill the following header convention (if in doubt, please let
        % Falcon.m create the function interface for you):
        %   Cost = FunctionHandle(outputs, states, controls, parameters);
        %   Cost: a scalar cost value
        %   outputs: column vector of outputs, if the model has some
        %   (otherwise this input is omitted)
        %   states: column vector of states
        %   controls: column vector of controls, if the model has some
        %   (otherwise this input is omitted)
        %   parameters: column vector of parameters, that were set by
        %   setParameters method (otherwise the input is omitted)
        % > Cost: The Cost object defining the name of the output.
        % > Phase..: The phases where the data for this PointConstraint should be
        % taken from.
        % > NormalizedTime..: The normalized time points where the data should be
        % taken. Each input phase requires its own normalized time vector.
        %  
        % <Keywords>
        % Problem!Cost!Mayer
        end

        function OpenPlotGUI(problemOrStruct)
        % Opens the plot graphical user interface with the given problem.
        %  
        % <Syntax>
        % falcon.Problem.OpenPlotGUI(problem);
        % falcon.Problem.OpenPlotGUI(structure);
        %  
        % <Description>
        % Creates a new instance of the plot GUI using the given problem. In the
        % plot GUI, the user can select all available time histories in the problem
        % and create plots from them.
        %  
        % <Inputs>
        % > problem: The falcon.Problem or a struct created from a problem (Using the
        % ToStruct() method) to be plotted.
        %  
        % <Keywords>
        % Problem!GUI!Open
        end

        function [cost, pointFunction] = addNewOutputCost(self, varargin)
        % 
        end

        function map = byNameUnique(self)
        % Get a struct for name based access to the object array, allowing
        % only unique names.
        %  
        % <Syntax>
        % [ map ] = obj.byNameUnique()
        %  
        % <Description>
        % The method returns all entries from the object array 'obj' as
        % fields of a struct, with the object Name as field name.  Throws
        % an error in case there are any duplicate names.
        %  
        % <Keywords>
        % OVC!byName!Unique
        end

        function map = byName(self)
        % Get a struct for name based access to the object array.
        %  
        % <Syntax>
        % [ map ] = obj.byName()
        %  
        % <Description>
        % The method returns all entries from the object array 'obj' as
        % fields of a struct, with the object Name as field name.  In case
        % there are duplicate names, the output struct contains object
        % arrays.
        %  
        % <Keywords>
        % OVC!byName
        end

        function computeParameterStatistics(obj, varargin)
        % COMPUTEPARAMETERSTATISTICS computes statistical properties of
        %   the parameter estimates.
        %  
        %  	<Syntax>
        %  	computeParameterStatistics( obj )
        %  
        %  	<Description>
        %  	after an estimation has be conducted, estimates for the
        %  	statistical properties of the parameter estimates
        %  	(covariances, correlations, standard deviations) are computed
        %  	using this method. In order to visualize them, the respective
        %  	plot methods of the falcon.parest.Parameter objects can be
        %  	used (e.g. plotCorrelationMatrix,
        %  	plotRelativeStandardDeviation)
        %  
        %  	<Inputs>
        %  	>obj: the Problem object
        %  
        %  	<ChangeLog>
        %  	2017-01-24 - Ch. Goettlicher: initial design
        %  	2017-12-19 - Ch. Goettlicher: commenting (-->EMT completion)
        end

        function updateSolverEvolution(obj, iterCount, info)
        % falcon.parest.core.UncertaintyComputation/updateSolverEvolution is a function.
        %   updateSolverEvolution(obj, iterCount, info)
        end

        function extractConstraintInformation(obj)
        % falcon.parest.core.ConstraintInformation/extractConstraintInformation is a function.
        %   extractConstraintInformation(obj)
        end

        function setSolver(obj, Solver)
        % falcon.parest.core.ConstraintInformation/setSolver is a function.
        %   setSolver(obj, Solver)
        end

    end

end