classdef (Abstract) TVC < falcon.core.Handle & matlab.mixin.CustomDisplay
    % FALCON.TRIM.CORE.TVC trim value container
    %  
    %  	<Description>
    %  	this class implements all functionalities, which are the same for
    %  	states, controls, outputs, and parameters
    %  
    %  	<ChangeLog>
    %  	2017-12-20 - Ch. Goettlicher: final commenting

    % -------------------------------------------------------------------------
    %                                FALCON.m
    % Copyright (c) 2014-2019 Institute of Flight System Dynamics, TUM, Munich
    % Matthias Bittner, Matthias Rieck, Maximilian Richter,
    % Benedikt Grueter, Johannes Diepolder, Florian Schwaiger,
    % Patrick Piprek, and Florian Holzapfel
    % Downloading, using, copying, or modifying FALCON.m code constitutes an
    % agreement to ALL of the terms of the FALCON.m EULA.
    % -------------------------------------------------------------------------

    properties
        % flag indicating if this value is known
        Known
        % the trim value
        TrimValue
        % trim grid
        TrimGrid
    end

    methods
        function fromModel(ModelHandle)
        % abstract function to get TVC information from model
        end

        function fromStruct(s)
        % abstract function to get TVC information from struct
        end

        function toFile(obj, FileName)
        % TOFILE store data in file
        %  
        %  	<Syntax>
        %  	toFile( obj, FileName )
        %  
        %  	<Description>
        %  	store the TVC data in a struct, which is then stored in a
        %  	file
        %  
        %  	<Inputs>
        %  	>obj: the TVC object
        %  	>FileName: the name of the file, from which the data is to
        %  	be loaded
        %  
        %  	<ChangeLog>
        %  	2017-03-10 - Ch. Goettlicher: initial design
        %  	2017-12-20 - Ch. Goettlicher: final commenting
        end

        function s = toTrimStruct(obj)
        % TOTRIMSTRUCT Create a struct from this object.
        %  
        %  	<Syntax>
        %  	[ s ] = toTrimStruct( obj )
        %  
        %  	<Description>
        %  	This metod creates a struct of this object including the
        %  	fields of the parent OVC and this TVC properties
        %  
        %  	<Inputs>
        %  	>obj: the TVC object
        %  
        %  	<Outputs>
        %  	>s: struct containing the inherent properties of this object.
        %  
        %  	<ChangeLog>
        %  	2017-03-10 - Ch. Goettlicher: initial design
        %  	2017-12-20 - Ch. Goettlicher: final commenting
        end

        function value = getGridValue(obj, idx)
        % GETGRIDVALUE get values corresponding to serial idx
        %  
        %  	<Syntax>
        %  	[ value ] = getGridValue( obj, idx )
        %  
        %  	<Description>
        %  	get the values from the grid in serialized fashion; output
        %  	array, if an array of objects is used for the call
        %  
        %  	<Inputs>
        %  	>obj: the TVC object
        %  	>idx: linear indices to extract from the grid
        %  
        %  	<Outputs>
        %  	>value: extracted values
        %  
        %  	<ChangeLog>
        %  	2017-03-10 - Ch. Goettlicher: initial design
        %  	2017-12-20 - Ch. Goettlicher: final commenting
        end

        function set(obj, name, prop, val)
        % SET certain property by name
        %  
        %  	<Syntax>
        %  	set( obj, name, prop, val )
        %  
        %  	<Description>
        %  	use this method to set a property in an array of TVCs by name
        %  
        %  	<Inputs>
        %  	>obj: 
        %  	>name: name of the TVC to manipulate
        %  	>prop: the name of the property to be set
        %  	>val: the value to be set
        %  
        %  	<ChangeLog>
        %  	2017-03-08 - Ch. Goettlicher: initial design
        %  	2017-12-20 - Ch. Goettlicher: final commenting
        end

        function GridPlot(obj, varargin)
        % plot grid values for this TVC value
        %  
        %   <Syntax>
        %   obj.GridPlot(XValue, 'YValues', YValues)
        %   obj.GridPlot(XValue, 'PlotArguments', PlotArguments)
        %   obj.GridPlot(XValue, 'RemainingBreakpoints', RemainingBreakpoints)
        %  
        %  	<Description>
        %  	plot the values of this TVC over the grid breakpoints. The
        %  	first argument is the x-values over which this TVC is to be
        %  	plotted. The other values have either to be fixed via
        %  	'RemainingBreakpoints' (using the name of the breakpoint as
        %  	Name in the Name-Value pair). Then the closest index in the
        %  	breakpoint vector is chosen. Or, it can be set via 'YValues',
        %  	in which case a three dimensional plot is created, with the
        %  	values of this TVC on the z-axis.
        %  
        %  	<Inputs>
        %  	>obj: 
        %  	>varargin: 
        %  	>XValue: value over which to plot (e.g. altitude, speed,
        %  	etc.)
        %  
        %  	<NameValue>
        %  	>YValues: if this is value is included, a 3D plot is
        %  	generated using XValue, YValues and the grid values on the
        %  	z-axis
        %  	>PlotArguments: cell array of arguments, that are directly
        %  	passed to the underlying plot function
        %  	>RemainingBreakpoints: names of remaining breakpoints to
        %  	fully define a point in the underlying ND-grid
        %  
        %  	<ChangeLog>
        %  	2017-03-08 - Ch. Goettlicher: initial design
        %  	2017-12-20 - Ch. Goettlicher: final commenting
        end

        function obj = fromFile(FileName)
        % FROMFILE load TVC data from file
        %  
        %   <Syntax>
        %   obj = falcon.trim.core.TVC.fromFile( FileName)
        %  
        %  	<Description>
        %  	load TVC data from file
        %  
        %  	<Inputs>
        %  	>FileName: Filename from which to load the data
        %  
        %  	<Outputs>
        %  	>obj: the newly created TVC object
        %  
        %  	<ChangeLog>
        %  	2017-03-13 - Ch. Goettlicher: initial design
        %  	2017-12-20 - Ch. Goettlicher: final commenting
        end

        function obj = TVC()
        % FALCON.TRIM.CORE.TVC trim value container
        %  
        %  	<Description>
        %  	this class implements all functionalities, which are the same for
        %  	states, controls, outputs, and parameters
        %  
        %  	<ChangeLog>
        %  	2017-12-20 - Ch. Goettlicher: final commenting
        end

    end

end