% build pt2 model using simulation model builder from falcon.m

clear
clear mex

falcon.init();

% model name
ModelName = 'Pt2_withConstants';

% get paths
currentDirectory = pwd();
ModelDirectory = fileparts(mfilename('fullpath'));
cd(ModelDirectory);

%% define states / controls / parameters / outputs
states   = [...
    falcon.State('x');
    falcon.State('v');
    ];

controls = [...
    falcon.Control('x_cmd');
    ];

parameters = [...
    ];

outputs = [...
    falcon.Output('y1'); ...
    falcon.Output('y2'); ...
    falcon.Output('v');
    ];

%% initialize model
mdl_build = falcon.parest.SimulationModelBuilder(...
    ModelName, ...
    states, ...
    controls, ...
    parameters, ...
    'Optimize', true, ...
    'DoDependencyCheck', false);

mdl_build.setOutputs( outputs );

mdl_build.addConstant( 'omega_0', 10);
mdl_build.addConstant( 'zeta', 0.2);
mdl_build.addConstant( 'K', 4);
mdl_build.addConstant( 'C1', 0.5);
mdl_build.addConstant( 'C2', 1 );


mdl_build.setStateDerivativeNames({...
    'x_dot', ...
    'v_dot'});

%% add dynamics
mdl_build.addSubsystem(Pt2SystemDynamic,...
    'Outputs',{'x_dot', 'v_dot'},...
    'Inputs', {'x', 'v', 'x_cmd', 'omega_0', 'zeta', 'K'});

%% add outputs
mdl_build.addSubsystem(Pt2Output, ...
    'Outputs',{'y1', 'y2'},...
    'Inputs', {'x', 'v', 'omega_0', 'zeta', 'C1', 'C2'});

%% compile model

mdl_build.Build('EvaluationProvider', 'mex');

cd(currentDirectory)

%% test
omega_0 = 10;
zeta = 0.2;
K = 4;

p = [
    omega_0;
    zeta;
    K;
    ];

% time step
delta_t = 0.01;

% define experiment inputs
t_end = 10;
t = 0:delta_t:t_end(1);

N = length(t);

u = zeros(1, N(1));
u( t>2 & t<=4) = 1;
u( t>4 & t<6) = -1;

x_0(1) = 8;
v_0(1) = 10;

state_0 = [x_0; v_0];
% [x, y] = Pt2_withConstants_sim(t, state_0, u, p);
[x, y, j_x, j_y] = Pt2_withConstants(state_0, u(:,1));

% [x, y] = Pt2_withConstants_sim(t, state_0, u, p);
[x, y] = Pt2_withConstants_sim(t, state_0, u);

% [x, y, Sx, Sy] = Pt2_withConstants_simSens(t, state_0, u, p);
[x, y, Sx, Sy] = Pt2_withConstants_simSens(t, state_0, u);

nSubplots = size(x,1)+size(y,1);
for iX = 1:size(x,1)
    subplot(nSubplots, 1, iX)
    plot(t, x(iX,:))
end
for iY = 1:size(y,1)
    subplot(nSubplots, 1, iY+size(x,1))
    plot(t, y(iY,:), 'r')
end