% introduction example to using falcon.m in system identification.
% estimate parameters of a second order lag model.
%
% the artificial measurement data for this example is created using the
% script generateDataPt2.m

falcon.init();

if ~exist('./Data/data_Pt2.mat', 'file')
    generateDataPt2();
end

close all
clear
clc

falcon.init();

%% add necessary paths - use relative paths!
BasePath = fileparts(mfilename('fullpath'));
addpath( fullfile(BasePath, 'Model') )
addpath( fullfile(BasePath, 'Data') )

% setup model - i.e. build anew, if it has changed
% buildPt2Model

%% ************************************************************************
% Step:    
% define ID states, controls, outputs, and parameters
% *************************************************************************

states   = [...
    falcon.parest.State('x');
    falcon.parest.State('v');
    ];

controls = [...
    falcon.parest.Control('x_cmd');
    ];

IdParameters = [...
    falcon.parest.Parameter('omega_0', 'Value',  10, 'Fixed', false, 'LowerBound', 0.001, 'Bayesian', false);...
    falcon.parest.Parameter(   'zeta', 'Value', 0.2, 'Fixed', false, 'LowerBound', 0.001, 'Bayesian', false);
    falcon.parest.Parameter(      'K', 'Value',   4, 'Fixed', false, 'LowerBound', 0.0001);
    falcon.parest.Parameter(     'C1', 'Value', 0.5, 'Fixed', true, 'LowerBound', 0);
    falcon.parest.Parameter(     'C2', 'Value',   1, 'Fixed', true, 'LowerBound', 0);
    ];
p_true = [IdParameters.Value]; 
IdParameters.setVariances( (diag( [10 0.2 nan nan nan] ) * 2e-2).^2 );

outputs = [...
    falcon.parest.Output('y1', 'Scaling', 1); ...
    falcon.parest.Output('y2', 'Scaling', 1); ...
    ];

% use randomly distorted version of true values for non-fixed parameters 
rng(42); % set random number generator to a defined state
% IdParameters(~isFixed).setValue( 7*rand(size(IdParameters(~isFixed))).* [IdParameters(~isFixed).Value].' );
isFixed = [IdParameters.isFixed];

% IdParameters(~isFixed).setValue( 0.001*ones(size(IdParameters(~isFixed))));
%% ************************************************************************
% Step:    
% load and configure experiments
% *************************************************************************

experiments = falcon.parest.Experiment.FromFile('data_Pt2.mat');

% configure experiments:
%   * set initial conditions: value & fixed / to be estimated
experiments(1).x0 = [...
    falcon.parest.Parameter('x1', 'Value',  8, 'Fixed', false); ...
    falcon.parest.Parameter('x2', 'Value', 10, 'Fixed', false); ...
    ];
% experiments(2).x0 = [4; -20];
experiments(2).x0.setFixed( false );

% configure experiments: 
%   * use less samples for faster computation (80%, 70%)
experiments(1).SampleTimeFactor = 0.3;
experiments(2).SampleTimeFactor = 0.4;

%% ************************************************************************
% Step:    
% setup estimation problem
% *************************************************************************

% initialize problem
ParameterEstimationProblem = falcon.parest.Problem('Pt2Estimation');
% add model, states, controls, parameters, outputs
ParameterEstimationProblem.addModel( @Pt2, IdParameters, states, controls, outputs );

%% ************************************************************************
% Step:    
% add experiments to problem
% *************************************************************************

ParameterEstimationProblem.addNewExperiment( experiments );
% generate initial guess for states

% % example filter data
% P_xx_0 = eye(2);
% Q = diag([1, 1]*1e2);
% R = diag([1, 1]*1e-4);
% E = zeros(2,2);
% % use EKF to obtain initial guess for states for experiment 1
% experiments.setFilterData(Q, R, P_xx_0);
% experiments(1).makeStateInitialGuess('filter'); 
% % use forward simulation for experiment 2
% experiments(2).makeStateInitialGuess('filter'); 

% use smooth measurement values as initial guess
C = diag([IdParameters(end-1:end).Value]);
for iExp = experiments(:).'
    z = iExp.getMeasurementValues();
    x = z{1};
    for iX = 1:size(z{1}, 1)
        x(iX, :) = smooth( x(iX, :), 20 )';
    end
    x = C \ x;
    
    iExp.makeStateInitialGuess('set', x);
end

%% ************************************************************************
% Step:    
% add cost function to problem
% *************************************************************************

costfunction = falcon.parest.costfunction.MaximumLikelihood();
% costfunction = falcon.parest.costfunction.MaximumLikelihood_mod();
% costfunction = falcon.parest.costfunction.LeastSquares();
costfunction.UpdatePlots = 1;   % update plots every x iterations
costfunction.ScalarScalingMode = 'auto_J';  
% costfunction.ScalarScalingMode = 'auto_dJ_dy';  
costfunction.J_y.CovarianceScalingMatrix = 'samples';  
costfunction.J_y.CovarianceScalingMatrix = 'off';  

% costfunction.addParameterCost('LeastSquares');

ParameterEstimationProblem.setCostFunction( costfunction );

%% ************************************************************************
% Step:    
% setup plots
% *************************************************************************

figure('units', 'normalized', 'outerposition', [0 0 1 1])
% plot experiments
experiments.plotExperiment('States', {'x', 'v'});

% configure and plot initial guess 
for iExperiment = 1:length(experiments)
    experiments(iExperiment).PlotCfg_model.Color = 'g';
end
experiments.plotModelOutput();

% configure and prepare progress plot
for iExperiment = 1:length(experiments)
    experiments(iExperiment).PlotCfg_model.Color = 'r';
end
experiments.plotModelOutput();

drawnow

%% ************************************************************************
% Step:    
% choose and configure solver and solve
% *************************************************************************

% ParameterEstimationProblem.improveInitialModelParameters('useInitialStateGuess', 'PrintLevel', 5); 
% ParameterEstimationProblem.improveInitialModelParameters('EEM', 'PrintLevel', 5); 

% ----------------- uncomment for IPOPT solution
optimizer = falcon.solver.ipopt();
% optimizer = falcon.solver.snopt();
ParameterEstimationProblem.Bake()
% figure; ParameterEstimationProblem.CheckGradient()

% % ----------------- uncomment for ML & Gauss-Newton solution
% % OptimizationStepAlgorithm = falcon.parest.optimization.GaussNewton;
% OptimizationStepAlgorithm = falcon.parest.optimization.LevenbergMarquardt;
% optimizer = falcon.parest.solver( OptimizationStepAlgorithm );
% optimizer.ParEstSolverOptions.ParallelWorkers = 3;
% optimizer.ParEstSolverOptions.ParallelProfile = 'local';

% set problem for solver
optimizer.setProblem(ParameterEstimationProblem);

% configure optimizer
optimizer.Options.MajorIterLimit = 300;
optimizer.Options.needH          = 0;

% solve problem
% optimizer.checkDerivatives_J_p( 'visualize_dJ', false, 'visualize_ddJ', true );
optimizer.Solve();

%% ************************************************************************
% Step:    
% plot final results
% *************************************************************************

ParameterEstimationProblem.computeParameterStatistics('force', 'Integration');
Cov3 = IdParameters.getCovariance();

% compute parameter statistics
% ParameterEstimationProblem.computeParameterStatistics('NullspaceMethod', 'collocation');
ParameterEstimationProblem.computeParameterStatistics('NullspaceMethod', 'eigs');%'force', 'Integration');
Cov1 = IdParameters.getCovariance();
ParameterEstimationProblem.computeParameterStatistics('NullspaceMethod', 'collocation');
Cov2 = IdParameters.getCovariance();

% update model output plots
experiments.updateModelOutput();
experiments.plotUncertainties( 3 );

%%
x0 = [experiments.x0];
Parameters_total = [x0(:); IdParameters];

% set parameter labels
Parameters_total.setLabels( {'x_1 (Exp1)', 'x_2 (Exp1)', 'x_1 (Exp2)', 'x_2 (Exp2)', '\omega_0', '\zeta', 'K', 'C_1', 'C_2'} ); 
% IdParameters(1).setLabels( {'\omega_0'} );
% IdParameters(2).setLabels( {'\zeta'} ); 

% plot parameter statistics
figure('position', [20 500 1500 400])
subplot(1,2,1)
Parameters_total.plotRelativeStandardDeviation();
subplot(1,2,2)
Parameters_total.plotCorrelationMatrix();

% plot evolution of parameters
figure
Parameters_total.plotParameterEvolution( );

% plot various parameter bar plots
figure
IdParameters.plotBar('plotInitialValues', 1, ...
    'trueValues', p_true, 'nSigma', 2);

%% ************************************************************************
% Step:    
% store resulting parameters to disc
% *************************************************************************

IdParameters.ToFile( fullfile( fileparts(which('data_Pt2.mat')), 'parameters_Pt2') )